#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2019 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from . import oauth
from . import account
from . import base
from . import config
from . import event
from . import locale
from . import role
from . import search
from . import settings
from . import snapshot

from .oauth import *
from .account import Account
from .base import Base
from .config import Config
from .event import Event
from .locale import Locale
from .role import Role
from .search import Search
from .settings import Settings
from .snapshot import Snapshot



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2019 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import json
import zipfile

import appier

from appier_extras import base

class ExportPart(appier.Part):
    """
    Modular part class that provides an infra-structure for the individual
    export of models using a JSON exporting format.

    This parts uses a monkey patching approach so that the model classes
    are changed adding the new export links.
    """

    def __init__(self, *args, **kwargs):
        appier.Part.__init__(self, *args, **kwargs)

    def version(self):
        return base.VERSION

    def load(self):
        appier.Part.load(self)
        self._monkey_patch()

    def routes(self):
        return [
            (("GET",), "/export/models/<str:model>.json", self.model_json, None, True),
            (("GET",), "/export/models/<str:model>.zip", self.model_zip, None, True),
            (("GET",), "/export/models/<str:model>/<str:_id>.json", self.entity_json, None, True),
            (("POST",), "/api/export/models/<str:model>", self.model_import_api, None, True)
        ]

    @classmethod
    @appier.link(name = "JSON (Export)", context = True)
    def json_global(cls, view = None, context = None, absolute = False):
        return appier.get_app().url_for(
            "export.model_json",
            model = cls._under(),
            view = view,
            context = context,
            absolute = absolute
        )

    @classmethod
    @appier.link(name = "ZIP (Export)", context = True)
    def zip_global(cls, view = None, context = None, absolute = False):
        return appier.get_app().url_for(
            "export.model_zip",
            model = cls._under(),
            view = view,
            context = context,
            absolute = absolute
        )

    @classmethod
    @appier.operation(
        name = "Import JSON (Export)",
        parameters = (
            ("JSON File", "file", "file"),
            ("Empty source", "empty", bool, False)
        )
    )
    def import_json(cls, file, empty):
        def callback(model_d):
            if "_id" in model_d: del model_d["_id"]
            model = cls(model_d)
            model.save(validate = False, verify = False)

        if empty: cls.delete_c()
        cls._json_import(file, callback)

    @classmethod
    @appier.operation(
        name = "Import ZIP (Export)",
        parameters = (
            ("ZIP File", "file", "file"),
            ("Empty source", "empty", bool, False)
        )
    )
    def import_zip(cls, file, empty):
        if empty: cls.delete_c()

        with zipfile.ZipFile(file, "r") as open_zip:
            for name in open_zip.namelist():
                content = open_zip.read(name)
                cls.import_json(content, False)

    @appier.ensure(token = "admin", context = "admin")
    def model_json(self, model):
        model_c = self.owner.get_model(model)
        object = appier.get_object(
            alias = True,
            find = True,
            limit = 0
        )
        models = self.owner.admin_part._find_view(
            model_c,
            map = True,
            rules = False,
            **object
        )
        return models

    @appier.ensure(token = "admin", context = "admin")
    def model_zip(self, model):
        encoding = self.field("encoding", default = "utf-8")

        model_c = self.owner.get_model(model)
        object = appier.get_object(
            alias = True,
            find = True,
            limit = 0
        )
        models = self.owner.admin_part._find_view(
            model_c,
            map = True,
            rules = False,
            **object
        )

        prefix = "%s-%s" % (self.owner.name, model)

        # creates the in memory file that is going to be used for the storage
        # of the ZIP file that is going to be created
        zip_io = appier.legacy.BytesIO()

        try:
            # creates the ZIP file using the in memory file and then dumps the
            # JSON based data into the associated directory, notice that a proper
            # encoding operation may be required to have a bytes object
            models_s = json.dumps(models)
            if not appier.legacy.is_bytes(models_s): models_s = models_s.encode(encoding)
            zip_file = zipfile.ZipFile(zip_io, mode = "w", allowZip64 = True)
            try: zip_file.writestr("%s.json" % prefix, models_s)
            finally: zip_file.close()

            # seeks the in memory file back to the initial position
            # and then reads the complete set of contents from it
            zip_io.seek(0)
            data = zip_io.read()
        finally:
            zip_io.close()

        return data

    @appier.ensure(token = "admin", context = "admin")
    def entity_json(self, model, _id):
        model_c = self.owner.get_model(model)
        entity = model_c.get(
             map = True,
             rules = False,
            _id = self.get_adapter().object_id(_id)
        )
        return entity

    @appier.ensure(token = "admin", context = "admin")
    def model_import_api(self, model):
        object = appier.get_object()
        model_c = self.owner.get_model(model)
        items = object.get("items", [])
        for model_d in items:
            if "_id" in model_d: del model_d["_id"]
            model = model_c(model_d)
            model.save(validate = False, verify = False)

    @appier.link(name = "JSON")
    def json(self):
        return appier.get_app().url_for(
            "export.entity_json",
            model = self.__class__._under(),
            _id = self._id
        )

    def _monkey_patch(
        self,
        models = None,
        base_cls = None,
        class_methods = None,
        instance_methods = None
    ):
        # verifies that a set of model classes have been provided to the method
        # and if that's not the case assumes that the complete set of model classes
        # registers in the current application should be patched
        models = models or list(self.owner.models_l)

        # runs the default operation on the base class so that if no value is provided
        # the default (parent) Appier class is used as the root of the monkey patch
        base_cls = base_cls or appier.Model

        # runs the defaulting on the class methods, these methods are considered the
        # recommended ones to be used in the initial monkey patching
        class_methods = class_methods or (
            ExportPart.json_global,
            ExportPart.zip_global,
            ExportPart.import_json,
            ExportPart.import_zip
        )

        # defaults the instance methods so that the required ones are used for the
        # monkey patching operation (as expected)
        instance_methods = instance_methods or (
            ExportPart.json,
        )

        # iterates over the complete set of class methods to add to the base
        # model class, then instantiates them and set the as attributes
        for class_method in class_methods:
            bound_method = classmethod(class_method.__func__)
            setattr(base_cls, class_method.__name__, bound_method)

        # iterates over the "special" instance methods to be added and changes
        # them to comply with the new instance intrinsics
        for instance_method in instance_methods:
            setattr(base_cls, instance_method.__name__, instance_method)

        # iterates over the complete set of model classes that are going to
        # be patched by adding the class and instance methods
        for model in models:
            # in case the model is not a sub class of the base class for
            # the current operation (nothing to be done) continues the loop
            if not issubclass(model, base_cls): continue

            # iterates over the "special" instance methods to be added and schedules
            # them to be bounded once the model instance is created, this is done
            # by adding them to the extra methods sequence of the model class
            for instance_method in instance_methods:
                model._extra_methods.append(
                    (
                        instance_method.__name__,
                        instance_method.__func__ if hasattr(instance_method, "__func__") else instance_method
                    )
                )

            # invalidates the methods cache in the model class, so that the
            # next retrieval already includes the newly added methods
            if "_methods" in model.__dict__: del model.__dict__["_methods"]



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2019 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2019 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import math

SIZE_UNITS_LIST = (
    "B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"
)
""" The size units list that contains the complete set of
units indexed by the depth they represent """

SIZE_UNITS_LIST_S = (
    "B", "K", "M", "G", "T", "P", "E", "Z", "Y"
)
""" The simplified size units list that contains the complete set of
units indexed by the depth they represent """

SIZE_UNIT_COEFFICIENT = 1024
""" The size unit coefficient as an integer value, this is
going to be used in each of the size steps as divisor """

DEFAULT_MINIMUM = 1024
""" The default minimum value meaning that this is the
maximum value that one integer value may have for the
size rounding operation to be performed """

DEFAULT_PLACES = 3
""" The default number of places (digits) that are going
to be used for the string representation in the round
based conversion of size units to be performed """

def size_round_unit(
    size_value,
    minimum = DEFAULT_MINIMUM,
    places = DEFAULT_PLACES,
    reduce = True,
    space = False,
    justify = False,
    simplified = False,
    depth = 0
):
    """
    Rounds the size unit, returning a string representation
    of the value with a good rounding precision.
    This method should be used to round data sizing units.

    Note that using the places parameter it's possible to control
    the number of digits (including decimal places) of the
    number that is going to be "generated".

    :type size_value: int/float
    :param size_value: The current size value (in bytes).
    :type minimum: int
    :param minimum: The minimum value to be used.
    :type places: int
    :param places: The target number of digits to be used for
    describing the value to be used for output, this is going
    to be used to calculate the proper number of decimal places.
    :type reduce: bool
    :param reduce: If the final string value should be reduced
    meaning that right decimal zeros should be removed as they
    represent an extra unused value.
    :type space: bool
    :param space: If a space character must be used dividing
    the value from the unit symbol.
    :type justify: bool
    :param justify: If the size string value should be (right)
    justified important for properly aligned values in a table.
    :type simplified: bool
    :param simplified: If the simplified version of the units
    should be used instead of the longer one.
    :type depth: int
    :param depth: The current iteration depth value.
    :rtype: String
    :return: The string representation of the data size
    value in a simplified manner (unit).
    """

    # in case the current size value is acceptable (less than
    # the minimum) this is the final iteration and the final
    # string representation is going to be created
    if size_value < minimum:
        # calculates the maximum size of the string that is going
        # to represent the base size value as the number of places
        # plus one (representing the decimal separator character)
        size_s = places + 1

        # calculates the target number of decimal places taking
        # into account the size (in digits) of the current size
        # value, this may never be a negative number
        log_value = size_value and math.log10(size_value)
        digits = int(log_value) + 1
        places = places - digits
        places = places if places > 0 else 0

        # creates the proper format string that is going to
        # be used in the creation of the proper float value
        # according to the calculated number of places
        format = "%%.%df" % places

        # rounds the size value, then converts the rounded
        # size value into a string based representation
        size_value = round(size_value, places)
        size_value_s = format % size_value

        # forces the reduce flag when the depth is zero, meaning
        # that an integer value will never be decimal, this is
        # required to avoid strange results for depth zero
        reduce = reduce or depth == 0

        # in case the dot value is not present in the size value
        # string adds it to the end otherwise an issue may occur
        # while removing extra padding characters for reduce
        if reduce and not "." in size_value_s: size_value_s += "."

        # strips the value from zero appended to the right and
        # then strips the value also from a possible decimal
        # point value that may be included in it, this is only
        # performed in case the reduce flag is enabled
        if reduce: size_value_s = size_value_s.rstrip("0")
        if reduce: size_value_s = size_value_s.rstrip(".")

        # in case the justify flag is set runs the justification
        # process on the size value taking into account the maximum
        # size of the associated size string
        if justify: size_value_s = size_value_s.rjust(size_s)

        # retrieves the size unit (string mode) for the current
        # depth according to the provided map
        if simplified: size_unit = SIZE_UNITS_LIST_S[depth]
        else: size_unit = SIZE_UNITS_LIST[depth]

        # retrieves the appropriate separator based
        # on the value of the space flag
        separator = space and " " or ""

        # creates the size value string appending the rounded
        # size value string and the size unit and returns it
        # to the caller method as the size value string
        size_value_string = size_value_s + separator + size_unit
        return size_value_string

    # otherwise the value is not acceptable and a new iteration
    # must be ran with one less depth of size value
    else:
        # re-calculates the new size value, increments the depth
        # and runs the size round unit again with the new values
        new_size_value = float(size_value) / SIZE_UNIT_COEFFICIENT
        new_depth = depth + 1
        return size_round_unit(
            new_size_value,
            minimum = minimum,
            places = places,
            reduce = reduce,
            space = space,
            justify = justify,
            simplified = simplified,
            depth = new_depth
        )
